/* 
 * SPDX-License-Identifier: MIT OR Apache-2.0
 *
 * Copyright (c) 2023 SUSE LLC
 *
 * Author: Roy Hopkins <rhopkins@suse.de>
 */


#ifndef IGVM_DEFS_H
#define IGVM_DEFS_H

/* Generated with cbindgen:0.28.0 */

/* This file is generated using cbindgen. Do not edit this file directly. */

#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

typedef struct {
    uint32_t is_2mb_page : 1;
    uint32_t unmeasured : 1;
    uint32_t reserved : 30;
} IgvmPageDataFlags;

typedef struct {
    uint32_t vtl2_protectable : 1;
    uint32_t reserved : 31;
} IgvmRequiredMemoryFlags;

typedef struct {
    uint16_t is_shared : 1;
    uint16_t reserved : 15;
} IgvmMemoryMapEntryFlags;

typedef struct {
    uint32_t memory_is_shared : 1;
    uint32_t reserved : 31;
} IgvmEnvironmentInfo;

typedef uint64_t IGVM_U64_LE;
typedef uint32_t IGVM_U32_LE;

#define IGVM_UINT32_FLAGS_VALUE(x) *((uint32_t*)&(x))


/**
 * IGVM format version 1.
 */
#define IGVM_FORMAT_VERSION_1 1

/**
 * IGVM format version 2. This is work-in-progress and not stabilized.
 */
#define IGVM_FORMAT_VERSION_2 2

/**
 * A page size of 4K (4096) bytes.
 */
#define IGVM_PAGE_SIZE_4K 4096

/**
 * Platform version for [`IgvmPlatformType::NATIVE`].
 */
#define IGVM_NATIVE_PLATFORM_VERSION 1

/**
 * Platform version for [`IgvmPlatformType::VSM_ISOLATION`].
 */
#define IGVM_VSM_ISOLATION_PLATFORM_VERSION 1

/**
 * Platform version for [`IgvmPlatformType::SEV_SNP`].
 */
#define IGVM_SEV_SNP_PLATFORM_VERSION 1

/**
 * Platform version for [`IgvmPlatformType::TDX`].
 */
#define IGVM_TDX_PLATFORM_VERSION 1

/**
 * Platform version for [`IgvmPlatformType::SEV`].
 */
#define IGVM_SEV_PLATFORM_VERSION 1

/**
 * Platform version for [`IgvmPlatformType::SEV_ES`].
 */
#define IGVM_SEV_ES_PLATFORM_VERSION 1

/**
 * This region describes VTL2.
 */
#define IGVM_VHF_RELOCATABLE_REGION_IS_VTL2 1

/**
 * The starting executable address for the specified VP and VTL should be
 * adjusted by the amount this region was relocated (RIP on x64, PC on arm64).
 */
#define IGVM_VHF_RELOCATABLE_REGION_APPLY_PC 2

/**
 * GDTR for the specified VP and VTL should be adjusted by the amount this
 * region was relocated. This is supported on X64 only.
 */
#define IGVM_VHF_RELOCATABLE_REGION_APPLY_GDTR 4

/**
 * Legacy name for ['IGVM_VHF_RELOCATABLE_REGION_APPLY_PC']
 */
#define IGVM_VHF_RELOCATABLE_REGION_APPLY_RIP IGVM_VHF_RELOCATABLE_REGION_APPLY_PC

/**
 * Page data types that describe the type of import for
 * [`IGVM_VHS_PAGE_DATA`].
 */
enum IgvmPageDataType {
  /**
   * Normal page data.
   */
  IGVM_PAGE_DATA_TYPE_NORMAL = 0,
  /**
   * Secrets page.
   * TODO: SEV-SNP only?
   */
  IGVM_PAGE_DATA_TYPE_SECRETS = 1,
  /**
   * CPUID data
   * TODO: SEV-SNP only?
   */
  IGVM_PAGE_DATA_TYPE_CPUID_DATA = 2,
  /**
   * CPUID_XF
   * SEV-SNP only?
   */
  IGVM_PAGE_DATA_TYPE_CPUID_XF = 3,
};
typedef uint16_t IgvmPageDataType;

/**
 * Enum describing different isolation platforms for
 * [`IGVM_VHS_SUPPORTED_PLATFORM`] structures.
 */
enum IgvmPlatformType {
  /**
   * Native platform type without isolation.
   */
  IGVM_PLATFORM_TYPE_NATIVE = 0,
  /**
   * Platform type of Hyper-V's which supports VSM isolation.
   */
  IGVM_PLATFORM_TYPE_VSM_ISOLATION = 1,
  /**
   * AMD SEV-SNP.
   */
  IGVM_PLATFORM_TYPE_SEV_SNP = 2,
  /**
   * Intel TDX.
   */
  IGVM_PLATFORM_TYPE_TDX = 3,
  /**
   * AMD SEV.
   */
  IGVM_PLATFORM_TYPE_SEV = 4,
  /**
   * AMD SEV-ES
   */
  IGVM_PLATFORM_TYPE_SEV_ES = 5,
};
typedef uint8_t IgvmPlatformType;

/**
 * The type of each structure in the variable header section.
 *
 * The top bit of this type may be set to indicate a loader may safely ignore
 * that structure.
 */
enum IgvmVariableHeaderType {
  /**
   * Invalid.
   */
  IGVM_INVALID = 0,
  /**
   * A supported platform structure described by
   * [`IGVM_VHS_SUPPORTED_PLATFORM`].
   */
  IGVM_VHT_SUPPORTED_PLATFORM = 1,
  /**
   * The isolation architecture policy for the guest, as defined by
   * [`IGVM_VHS_GUEST_POLICY`].
   */
  IGVM_VHT_GUEST_POLICY = 257,
  /**
   * A relocatable region structure described by
   * [`IGVM_VHS_RELOCATABLE_REGION`].
   */
  IGVM_VHT_RELOCATABLE_REGION = 258,
  /**
   * A page table relocation region described by
   * [`IGVM_VHS_PAGE_TABLE_RELOCATION`].
   */
  IGVM_VHT_PAGE_TABLE_RELOCATION_REGION = 259,
  /**
   * A parameter area structure described by [`IGVM_VHS_PARAMETER_AREA`].
   */
  IGVM_VHT_PARAMETER_AREA = 769,
  /**
   * A page data structure described by [`IGVM_VHS_PAGE_DATA`].
   */
  IGVM_VHT_PAGE_DATA = 770,
  /**
   * A parameter insert structure described by
   * [`IGVM_VHS_PARAMETER_INSERT`].
   */
  IGVM_VHT_PARAMETER_INSERT = 771,
  /**
   * A VP context structure described by [`IGVM_VHS_VP_CONTEXT`].
   */
  IGVM_VHT_VP_CONTEXT = 772,
  /**
   * A required memory structure described by
   * [`IGVM_VHS_REQUIRED_MEMORY`].
   */
  IGVM_VHT_REQUIRED_MEMORY = 773,
  /**
   * Was previously used in earlier revisions as
   * `IGVM_VHT_SHARED_BOUNDARY_GPA` but is now unused. Do not use for
   * future revisions.
   */
  IGVM_RESERVED_DO_NOT_USE = 774,
  /**
   * A parameter which holds a 4-byte u32 value defining the number of
   * VPs that the host intends to start. The loader will write the VP
   * count to the specified offset of the specified parameter area. The
   * parameter location information is specified by a structure of type
   * [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_VP_COUNT_PARAMETER = 775,
  /**
   * A parameter where the loader will deposit the ACPI SRAT
   * table describing topology information as specified by a structure of
   * type [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_SRAT = 776,
  /**
   * A parameter where the loader will deposit the ACPI MADT table
   * describing CPU topology information as specified by a structure of
   * type [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_MADT = 777,
  /**
   * A parameter where the loader will deposit the mmio ranges of the
   * guest as specified by [`IGVM_VHS_PARAMETER`].
   *
   * The format of the MMIO ranges written by the loader are defined as
   * [`IGVM_VHS_MMIO_RANGES`].
   */
  IGVM_VHT_MMIO_RANGES = 778,
  /**
   * The AMD SEV-SNP ID block to be used described by
   * [`IGVM_VHS_SNP_ID_BLOCK`].
   */
  IGVM_VHT_SNP_ID_BLOCK = 779,
  /**
   * A parameter where the loader will deposit the memory available to the
   * guest as specified by [`IGVM_VHS_PARAMETER`].
   *
   * The format of the memory information written by the loader will be
   * an array of [`IGVM_VHS_MEMORY_MAP_ENTRY`] structures.
   */
  IGVM_VHT_MEMORY_MAP = 780,
  /**
   * An error range structure described by [`IGVM_VHS_ERROR_RANGE`].
   */
  IGVM_VHT_ERROR_RANGE = 781,
  /**
   * A parameter which describes a null terminated ASCII command line to
   * be used with the guest as specified by [`IGVM_VHS_PARAMETER`]. The
   * format of the command line is specific to the guest being loaded.
   *
   * For example, this parameter could be used to describe the Linux
   * kernel command line.
   *
   * In the [`IgvmVariableHeaderType::IGVM_VHT_DEVICE_TREE`] parameter,
   * this corresponds to the `/chosen/bootargs` field.
   */
  IGVM_VHT_COMMAND_LINE = 782,
  /**
   * A parameter where the loader will deposit the ACPI SLIT table
   * describing topology information as specified by a structure of type
   * [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_SLIT = 783,
  /**
   * A parameter where the loader will deposit the ACPI PPTT table
   * describing processor topology information as specified by a
   * structure of type [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_PPTT = 784,
  /**
   * A VBS measurement structure described by
   * [`IGVM_VHS_VBS_MEASUREMENT`].
   */
  IGVM_VHT_VBS_MEASUREMENT = 785,
  /**
   * A parameter which contains a full description of the guest partition
   * in the Flattened Device Tree (DTB) format, as defined by the Device
   * Tree specification. The intention of this parameter is to supersede
   * all previous parameter types, and allow for the host to report new
   * additional information without requiring a new IGVM parameter type,
   * and a corresponding IGVM specification update. Thus, a guest should
   * be able to get all host parameter information by creating an IGVM
   * file with this parameter only. No new IGVM parameter types should be
   * added.
   *
   * Device tree definitions follow standard definitions in the device
   * tree specification, or Linux definitions defined in the kernel tree
   * under device tree bindings for specific devices.
   *
   * IGVM specific extensions can be found in the [`dt`] module.
   */
  IGVM_VHT_DEVICE_TREE = 786,
  /**
   * A parameter which holds a u32 bitfield value defining environmental
   * state of the VM.  The bitfield is defined by the `IgvmEnvironmentInfo`
   * structure.  The loader will write the state to the specified offset of
   * the specified parameter area.  The parameter location information is
   * specified by a structure of type [`IGVM_VHS_PARAMETER`].
   */
  IGVM_VHT_ENVIRONMENT_INFO_PARAMETER = 787,
};
typedef uint32_t IgvmVariableHeaderType;

/**
 * The type of memory described by a memory map entry or device tree node.
 */
enum IgvmMemoryMapEntryType {
  /**
   * Normal memory.
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_MEMORY = 0,
  /**
   * Platform reserved memory.
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_PLATFORM_RESERVED = 1,
  /**
   * Persistent memory (PMEM).
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_PERSISTENT = 2,
  /**
   * Memory where VTL2 protections that deny access to lower VTLs can be
   * applied. Some isolation architectures only allow VTL2 protections on
   * certain memory ranges.
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_VTL2_PROTECTABLE = 3,
  /**
   * Specific Purpose memory (SPM). This is memory with special properties
   * reserved for specific purposes and shouldn't be used by the firmware
   * or operating system. This corresponds with the UEFI memory map entry
   * flag EFI_MEMORY_SP, introduced in UEFI 2.8.
   * See https://uefi.org/specs/UEFI/2.10/07_Services_Boot_Services.html
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_SPECIFIC_PURPOSE = 4,
  /**
   * Hidden memory is visible in the memory map but is hidden from any other
   * enumeration that may be used to expose available memory to the VM.
   */
  IGVM_MEMORY_MAP_ENTRY_TYPE_HIDDEN = 5,
};
typedef uint16_t IgvmMemoryMapEntryType;

/**
 * The AMD SEV-SNP Guest policy used in [`IGVM_VHS_GUEST_POLICY::policy`].
 *
 * For now, this matches the definition in Section 4.3 Guest Policy of the AMD
 * SEV-SNP specification.
 */
typedef struct IgvmSnpPolicy IgvmSnpPolicy;

/**
 * The Intel TDX policy used in [`IGVM_VHS_GUEST_POLICY::policy`].
 */
typedef struct IgvmTdxPolicy IgvmTdxPolicy;

/**
 * The version 1 fixed header that is at the start of every IGVM file.
 */
typedef struct IGVM_FIXED_HEADER {
  /**
   * A u32 that must hold the value described by [`IGVM_MAGIC_VALUE`].
   */
  uint32_t magic;
  /**
   * Describes the version of the file format that must be supported by the
   * loader in order for this file to load successfully.
   */
  uint32_t format_version;
  /**
   * Describes the byte offset to the variable header in the file. This must be
   * greater than or equal to the size of the fixed header, and must be
   * aligned to an 8-byte boundary.
   */
  uint32_t variable_header_offset;
  /**
   * Describes the total size in bytes of the variable header in the file.
   */
  uint32_t variable_header_size;
  /**
   * Describes the total size of the file in bytes.
   */
  uint32_t total_file_size;
  /**
   * Contains a CRC32 that spans the fixed and variable header, but not the
   * file data section. The checksum is calculated as if this checksum field
   * is zero.
   */
  uint32_t checksum;
} IGVM_FIXED_HEADER;

/**
 * The header describing each structure in the variable header section. Headers
 * are aligned to 8 byte boundaries.
 */
typedef struct IGVM_VHS_VARIABLE_HEADER {
  /**
   * The type of the header.
   */
  IgvmVariableHeaderType typ;
  /**
   * The length of the header. Note that this might not be 8 byte aligned,
   * but instead describes the size of the content within the structure.
   */
  uint32_t length;
} IGVM_VHS_VARIABLE_HEADER;

/**
 * This structure indicates which isolation platforms are compatible with this
 * guest image. A separate [`IGVM_VHS_SUPPORTED_PLATFORM`] structure must be
 * used for each compatible platform.
 *
 * The [`IGVM_VHS_SUPPORTED_PLATFORM`] structure must appear in the header
 * prior to any other structures that refer to the compatibility mask that this
 * structure defines.
 */
typedef struct IGVM_VHS_SUPPORTED_PLATFORM {
  /**
   * A bitmask that is used in following variable header structures that
   * correspond with this platform. Headers that have this corresponding bit
   * set indicates that it should be loaded if loading this specified
   * platform.
   *
   * This must have only one bit set.
   */
  uint32_t compatibility_mask;
  /**
   * Which VTL will be the highest VTL activated for the guest. On platforms
   * that do not support multiple VTLs, this value must be zero.
   */
  uint8_t highest_vtl;
  /**
   * Which platform is supported, as defined by [`IgvmPlatformType`].
   */
  IgvmPlatformType platform_type;
  /**
   * The platform version.
   */
  uint16_t platform_version;
  /**
   * This field describes the GPA at which memory above the boundary will be
   * host visible. A value of 0 indicates that this field is ignored, and the
   * platform described will manage shared memory in an enlightened manner.
   */
  uint64_t shared_gpa_boundary;
} IGVM_VHS_SUPPORTED_PLATFORM;

/**
 * This structure defines the guest policy that is isolation architecture
 * dependent.
 */
typedef struct IGVM_VHS_GUEST_POLICY {
  /**
   * The specified policy which is isolation architecture dependent.
   *
   * For AMD SEV-SNP, this is [`SnpPolicy`].
   *
   * For Intel TDX, this is [`TdxPolicy`].
   */
  uint64_t policy;
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * Reserved, must be zero.
   */
  uint32_t reserved;
} IGVM_VHS_GUEST_POLICY;

/**
 * Indicate a relocatable region. This region may be relocated according to the
 * fields within the header. The region must be relocated as a whole, with each
 * gpa within the region described being relocated by the same amount. Other
 * directive headers that specify gpas must be relocated by the amount the
 * region was relocated.
 *
 * The loader must guarantee memory is present for the whole region described
 * by the header.
 *
 * Additional flags specify behavior for registers and further information
 * about this region.
 *
 * # Architecture
 * X64 only.
 */
typedef struct IGVM_VHS_RELOCATABLE_REGION {
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * VP Index for register flags to be applied to.
   */
  uint16_t vp_index;
  /**
   * VTL for register flags to be applied to.
   */
  uint8_t vtl;
  /**
   * Additional flags that describe this region.
   */
  uint8_t flags;
  /**
   * The relocation alignment this region can be relocated by.
   *
   * Must be multiple of 4K.
   */
  uint64_t relocation_alignment;
  /**
   * The base guest physical address of this relocatable region.
   *
   * `relocation_region_gpa` must be aligned to `relocation_alignment`.
   */
  uint64_t relocation_region_gpa;
  /**
   * The overall size of this relocatable region. Must be 4K aligned.
   */
  uint64_t relocation_region_size;
  /**
   * The minimum address this section can be relocated to.
   *
   * Must be aligned to `relocation_alignment`.
   */
  uint64_t minimum_relocation_gpa;
  /**
   * Maximum address this section can be relocated to.
   *
   * Must be aligned to `relocation_alignment`.
   */
  uint64_t maximum_relocation_gpa;
} IGVM_VHS_RELOCATABLE_REGION;

/**
 * Indicate the region of memory containing the page table which can be
 * relocated. This region must contain the root page table (CR3 on x64, TTBR0
 * on arm64). Page table pages that are described that lay outside of this
 * region will not be walked nor fixed up. Page tables cannot lie within
 * other relocatable regions. Similar to [`IGVM_VHS_RELOCATABLE_REGION`], the
 * loader must guarantee memory is present for the whole region described by
 * this header.
 *
 * The VP index and VTL is used to describe the paging state for the page
 * table. The root address (CR3 on x64, TTBR0 on arm64) will be then fixed up
 * according to the amount this region was relocated by. Other VPs within the
 * same VTL that have the same initial page table address as the described VP
 * will also be fixed up. There must be a [`IGVM_VHS_VP_CONTEXT`] structure
 * for the given vp index and vtl. The given VP must be in 64 bit mode with
 * paging enabled.
 *
 * This region describes any additional pages the loader may use to allocate
 * additional entries to handle relocation. `used_size` represents the
 * allocated size starting from the base `gpa`.
 *
 * The page table must map the address space as identity mapped, with VA = PA.
 *
 * Initial page table data is specified by [`IGVM_VHS_PAGE_DATA`] headers that
 * are contained within the region described by this header. There must not be
 * any [`IGVM_VHS_PARAMETER_INSERT`] headers that refer to gpas described by
 * this header.
 *
 * Page directory entries pointing to another page table are fixed up according
 * to the amount this region is relocated by, if the PDE points to a page
 * within this region.
 *
 * Page table entries that map ranges that have been relocated are also further
 * fixed up. The entry must be moved to the new identity mapped VA
 *     corresponding to where the region was relocated to.
 *
 * For example:
 * - Region A is relocatable, and originally describes GPAs 0x1000 to 0x2000
 * - The page table contains a 1GB entry mapping 0x0 - 1GB.
 * - Region A is chosen to be relocated to a new base, 2GB.
 * - The page table entry previously identity mapping VA 0x0
 * - 1GB is now moved to 0x2GB - 0x3GB, which now identity maps 0x2GB - 0x3GB.
 *
 * However, page table entries that map a larger range than the relocation
 *     amount are not moved.
 *
 * For example:
 * - Region A is relocatable, and originally describes GPAs 0x1000 to 0x2000
 * - The page table contains a 1GB entry mapping 0x0 - 1GB.
 * - Region A is chosen to be relocated to a new base, 0x5000.
 * - The page table entry previously identity mapping VA 0x0 - 1GB still maps
 *   the relocated region, and is left as is mapping VA 0x0 - 1GB to PA 0x0 - 1GB.
 *
 * For regions that are relocated at a non-aligned amount greater than the PTE
 * mapping size, the IGVM file is considered invalid.
 *
 * For example:
 * - Region A is relocatable, and originally describes GPAs 0x0000 to 0x2000.
 * - The page table contains a 1GB entry mapping 0x0 - 1GB.
 * - Region A is chosen to be relocated to a new base, 1.5GB. This has a
 *   relocation offset of +1.5GB.
 * - The page table entry previously identity mapping VA 0x0 - 1GB needs to be
 *   relocated, but cannot be due to the the mapping now being unaligned. This
 *   IGVM file is invalid, as the relocation region A has an incorrect
 *   `relocation_alignment`.
 *
 * Page table entries that map ranges that have not been relocated are left as
 * is.
 *
 * # Architecture
 * X64 and ARM64.
 */
typedef struct IGVM_VHS_PAGE_TABLE_RELOCATION {
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * VP Index for paging information.
   * On X64 these registers include CR3, CR4 and EFER.
   * On aarch64 these registers include TCR and TTBR0.
   */
  uint16_t vp_index;
  /**
   * VTL for paging information.
   */
  uint8_t vtl;
  /**
   * Reserved, must be zero.
   */
  uint8_t reserved;
  /**
   * The base of the page table region.
   */
  uint64_t gpa;
  /**
   * The total size in bytes of the page table region.
   */
  uint64_t size;
  /**
   * The number of already used pages by the pagetable.
   */
  uint64_t used_size;
} IGVM_VHS_PAGE_TABLE_RELOCATION;

/**
 * This structure defines a region of memory that can be used for holding
 * parameters to be passed into the guest. Parameter areas are created with
 * this structure, then parameters may be imported via parameter structures
 * that define the byte offset to deposit the given parameter into a given
 * parameter area by `parameter_area_index`.
 *
 * The paramter area is imported into the guest address space via a
 * [`IGVM_VHS_PARAMETER_INSERT`] structure.
 */
typedef struct IGVM_VHS_PARAMETER_AREA {
  /**
   * The size of the parameter region, in bytes. This must be page aligned.
   */
  uint64_t number_of_bytes;
  /**
   * An index that will be used by other structure types that refer to a
   * parameter area. Each parameter area index can be used exactly once.
   */
  uint32_t parameter_area_index;
  /**
   * A file offset from which to load the initial contents of the parameter
   * page; these initial contents can be overwritten as parameters are
   * inserted according to other variable header structures which direct the
   * insertion of parameters. If `file_offset` is zero, then the initial
   * contents of the parameter page will be a page of zeroes.
   */
  uint32_t file_offset;
} IGVM_VHS_PARAMETER_AREA;

/**
 * This structure describes a page of data that should be loaded into the guest
 * address space.
 *
 * The VTL permissions of page data loaded by the loader must conform to the
 * architectural default page permissions. On some platforms, this may require
 * the highest VTL to make pages loaded by the loader accessible to lower VTLs.
 * The following table describes architectural defaults for VTL permissions:
 *
 * | Architecture              | VTL Access Permissions |
 * |---------------------------|------------------------|
 * | VBS                       | All VTLs               |
 * | SEV-SNP                   | Highest VTL only       |
 * | TDX partitioning disabled | All VTLs               |
 * | TDX partitioning enabled  | Highest VTL only       |
 *
 */
typedef struct IGVM_VHS_PAGE_DATA {
  /**
   * The guest physical address at which this page data should be loaded; it
   * must be aligned to a page size boundary.
   */
  uint64_t gpa;
  /**
   * The compatibility mask for this structure.
   */
  uint32_t compatibility_mask;
  /**
   * The `file_offset` field specifies the offset in bytes at which the page
   * data can be found. Note that this offset is relative to the beginning of
   * the file, not the start of the file data section.
   *
   * If `file_offset`` is zero, then a page of zeroes is to be loaded. Even
   * though certain platforms may support a native zero page type, the loader
   * should treat this page with a `file_offset` of zero as a normal private
   * page with zeros as contents. A future page type will be required for
   * specifying native hardware zero pages on supported platforms.
   */
  uint32_t file_offset;
  /**
   * Flags.
   */
  IgvmPageDataFlags flags;
  /**
   * The data type for this page data.
   */
  IgvmPageDataType data_type;
  /**
   * Reserved.
   */
  uint16_t reserved;
} IGVM_VHS_PAGE_DATA;

/**
 * This structure controls the insertion of a parameter area into the guest
 * address space.
 */
typedef struct IGVM_VHS_PARAMETER_INSERT {
  /**
   * The guest physical address where this parameter area should be imported
   * into the guest address space. It must be page size aligned.
   *
   * Note that this parameter area is imported as a set of unmeasured pages
   * when this structure is encountered by the loader, starting with the
   * lowest GPA.
   */
  uint64_t gpa;
  /**
   * The compatibility mask for this structure.
   */
  uint32_t compatibility_mask;
  /**
   * The parameter area index that should be imported. This must have been
   * declared by a previous [`IGVM_VHS_PARAMETER_AREA`] structure.
   *
   * Once a parameter area index is inserted, that index becomes invalid for
   * future use.
   */
  uint32_t parameter_area_index;
} IGVM_VHS_PARAMETER_INSERT;

/**
 * The common parameter structure used by different parameter types to indicate
 * to the loader which parameter area and what offset to deposit the specified
 * runtime info. See corresponding parameter types in
 * [`IgvmVariableHeaderType`].
 *
 * A well behaving loader should fail loading the IGVM file if a parameter does
 * not fit in the requested parameter area with the specified offset.
 *
 * However, because the values written are unmeasured and untrusted, isolated
 * guests must perform validation on the values written into parameter areas. A
 * malicious or buggy loader may write any values.
 */
typedef struct IGVM_VHS_PARAMETER {
  /**
   * The paramter area index for this parameter.
   */
  uint32_t parameter_area_index;
  /**
   * The byte offset within the parameter area to deposit this parameter.
   */
  uint32_t byte_offset;
} IGVM_VHS_PARAMETER;

/**
 * This structure defines architecture specific that should be loaded into the
 * guest address space to represent an initial VP context.
 *
 * A file can contain only a single [`IGVM_VHS_VP_CONTEXT`] entry.
 *
 * For architectures such as Intel TDX that have an architectural reset state,
 * this structure is invalid.
 *
 * Note that this structure is not aligned to 8 bytes.
 */
typedef struct IGVM_VHS_VP_CONTEXT {
  /**
   * The guest physical address where the VP context data should be
   * deposited, if required.
   *
   * For AMD SEV, NATIVE or VBS, this must be zero. There is no VP context data to be
   * deposited into the guest address space.
   *
   * For AMD SEV-ES or SEV-SNP, this is the GPA to place the VMSA structure at.
   */
  IGVM_U64_LE gpa;
  /**
   * The compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * The file offset for arch specific VP context data.
   *
   * For VBS, the format of VP state is defined by a list of registers
   * defined by [`VbsVpContextRegister`] structures prefixed by a
   * [`VbsVpContextHeader`] in the file data section.
   *
   * For AMD SEV-ES and SEV-SNP, the data is the form of the AMD SEV-SNP VMSA
   * structure.
   */
  uint32_t file_offset;
  /**
   * The VP index for this VP context.
   */
  uint16_t vp_index;
  /**
   * Reserved.
   */
  uint16_t reserved;
} IGVM_VHS_VP_CONTEXT;

/**
 * Format of VBS [`IGVM_VHS_VP_CONTEXT`] file data.
 *
 * The format consists of a [`VbsVpContextHeader`] followed by a
 * `register_count` of [`VbsVpContextRegister`].
 */
typedef struct IgvmVbsVpContextHeader {
  /**
   * The number of registers in this VP context.
   */
  uint32_t register_count;
} IgvmVbsVpContextHeader;

/**
 * The registers associated with a VBS [`IGVM_VHS_VP_CONTEXT`] structure in the
 * file data section.
 */
typedef struct IgvmVbsVpContextRegister {
  /**
   * The VTL to import this register to.
   */
  uint8_t vtl;
  /**
   * The Hyper-V register name as specified by the TLFS.
   */
  IGVM_U32_LE register_name;
  /**
   * Padding to align up to next 0x10 offset.
   */
  uint8_t mbz[11];
  /**
   * The value of the register, as a Hyper-V register value.
   */
  uint8_t register_value[16];
} IgvmVbsVpContextRegister;

/**
 * This structure describes memory the IGVM file expects to be present in the
 * guest. This is a hint to the loader that the guest will not function without
 * memory present at the specified range, and should terminate the load process
 * if memory is not present.
 *
 * This memory may or may not be measured, depending on the other structures
 * this range overlaps with in the variable header section.
 *
 * Note that the guest cannot rely on memory being present at this location at
 * runtime, as a malicious host may choose to ignore this header.
 */
typedef struct IGVM_VHS_REQUIRED_MEMORY {
  /**
   * The base guest physical address for this range. This must be page
   * aligned.
   */
  uint64_t gpa;
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * The number of bytes for this range. This must be page aligned.
   */
  uint32_t number_of_bytes;
  /**
   * Flags for this range, as defined by [`IgvmRequiredMemoryFlags`].
   */
  IgvmRequiredMemoryFlags flags;
  /**
   * Reserved.
   */
  uint32_t reserved;
} IGVM_VHS_REQUIRED_MEMORY;

/**
 * A structure describing memory via a range of pages.
 */
typedef struct IGVM_VHS_MEMORY_RANGE {
  /**
   * The base guest physical page number for this range.
   */
  uint64_t starting_gpa_page_number;
  /**
   * The number of pages in this range.
   */
  uint64_t number_of_pages;
} IGVM_VHS_MEMORY_RANGE;

/**
 * The format used to describe the MMIO ranges of the guest for a
 * [`IgvmVariableHeaderType::IGVM_VHT_MMIO_RANGES`] parameter.
 *
 * Note that this structure can only define two mmio ranges, for a full
 * reporting of the guest's mmio ranges, the
 * [`IgvmVariableHeaderType::IGVM_VHT_DEVICE_TREE`] parameter should be used
 * instead.
 */
typedef struct IGVM_VHS_MMIO_RANGES {
  /**
   * The mmio ranges for the guest. Note that this structure can only report
   * two ranges, regardless of how many are available to the guest.
   */
  struct IGVM_VHS_MEMORY_RANGE mmio_ranges[2];
} IGVM_VHS_MMIO_RANGES;

/**
 * Signature for SNP ID block. See the corresponding PSP definitions.
 */
typedef struct IGVM_VHS_SNP_ID_BLOCK_SIGNATURE {
  /**
   * r_comp
   */
  uint8_t r_comp[72];
  /**
   * s_comp
   */
  uint8_t s_comp[72];
} IGVM_VHS_SNP_ID_BLOCK_SIGNATURE;

/**
 * Public key for SNP ID block. See the corresponding PSP definitions.
 */
typedef struct IGVM_VHS_SNP_ID_BLOCK_PUBLIC_KEY {
  /**
   * curve
   */
  uint32_t curve;
  /**
   * Reserved.
   */
  uint32_t reserved;
  /**
   * qx
   */
  uint8_t qx[72];
  /**
   * qy
   */
  uint8_t qy[72];
} IGVM_VHS_SNP_ID_BLOCK_PUBLIC_KEY;

/**
 * This structure describes the AMD SEV-SNP ID block.
 *
 * AuthorKeyEnabled is set to 0x1 if an author key is to be used, with the
 * following corresponding author keys populated. Otherwise, the author key
 * fields must be zero.
 *
 * Other fields share the same meaning as defined in the SNP API specification.
 *
 * TODO: doc links for fields to SNP spec.
 */
typedef struct IGVM_VHS_SNP_ID_BLOCK {
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * author_key_enabled
   */
  uint8_t author_key_enabled;
  /**
   * reserved
   */
  uint8_t reserved[3];
  /**
   * ld
   */
  uint8_t ld[48];
  /**
   * family id
   */
  uint8_t family_id[16];
  /**
   * image id
   */
  uint8_t image_id[16];
  /**
   * version
   */
  uint32_t version;
  /**
   * guest svm
   */
  uint32_t guest_svn;
  /**
   * id key algorithm
   */
  uint32_t id_key_algorithm;
  /**
   * author key algorithm
   */
  uint32_t author_key_algorithm;
  /**
   * id key signature
   */
  struct IGVM_VHS_SNP_ID_BLOCK_SIGNATURE id_key_signature;
  /**
   * id public key
   */
  struct IGVM_VHS_SNP_ID_BLOCK_PUBLIC_KEY id_public_key;
  /**
   * author key signature
   */
  struct IGVM_VHS_SNP_ID_BLOCK_SIGNATURE author_key_signature;
  /**
   * author public key
   */
  struct IGVM_VHS_SNP_ID_BLOCK_PUBLIC_KEY author_public_key;
} IGVM_VHS_SNP_ID_BLOCK;

/**
 * This structure describes a VBS measurement to be used with Hyper-V's VBS
 * isolation architecture.
 *
 * TODO: doc fields
 */
typedef struct IGVM_VHS_VBS_MEASUREMENT {
  /**
   * Compatibility mask
   */
  uint32_t compatibility_mask;
  /**
   * version
   */
  uint32_t version;
  /**
   * product id
   */
  uint32_t product_id;
  /**
   * module id
   */
  uint32_t module_id;
  /**
   * security version
   */
  uint32_t security_version;
  /**
   * policy flags
   */
  uint32_t policy_flags;
  /**
   * boot digest algo
   */
  uint32_t boot_digest_algo;
  /**
   * signing algo
   */
  uint32_t signing_algo;
  /**
   * boot measurement digest
   */
  uint8_t boot_measurement_digest[64];
  /**
   * signature
   */
  uint8_t signature[256];
  /**
   * public key
   */
  uint8_t public_key[512];
} IGVM_VHS_VBS_MEASUREMENT;

/**
 * The structure deposited by the loader for memory map entries for
 * [`IgvmVariableHeaderType::IGVM_VHT_MEMORY_MAP`] that describe memory
 * available to the guest.
 *
 * A well-behaved loader will report these in sorted order, with a final entry
 * with `number_of_pages` with zero signifying the last entry.
 */
typedef struct IGVM_VHS_MEMORY_MAP_ENTRY {
  /**
   * The starting gpa page number for this range of memory.
   */
  uint64_t starting_gpa_page_number;
  /**
   * The number of pages in this range of memory.
   */
  uint64_t number_of_pages;
  /**
   * The type of memory this entry represents.
   */
  IgvmMemoryMapEntryType entry_type;
  /**
   * Flags about this memory entry.
   */
  uint16_t flags;
  /**
   * Reserved.
   */
  uint32_t reserved;
} IGVM_VHS_MEMORY_MAP_ENTRY;

/**
 * This structure describes a range in the GPA space of the guest that will be
 * reserved for reporting errors to the host. Therefore, the memory described
 * by this structure will also be marked as accessible by the host for the
 * lifetime of the VM.
 *
 * These pages are imported by the loader with shared visibility.
 *
 * The format of the data written by the guest into the error ranges to be read
 * by the host are outside of the scope of this specification.
 */
typedef struct IGVM_VHS_ERROR_RANGE {
  /**
   * The guest physical address that this error range starts at. It must be
   * page aligned.
   */
  uint64_t gpa;
  /**
   * Compatibility mask.
   */
  uint32_t compatibility_mask;
  /**
   * The size of this range in bytes. It must be page aligned.
   */
  uint32_t size_bytes;
} IGVM_VHS_ERROR_RANGE;

#endif  /* IGVM_DEFS_H */
